@@ -1,5 +1,5 @@
 //
-// Copyright (c) 2019-2023 Red Hat, Inc.
+// Copyright (c) 2019-2024 Red Hat, Inc.
 // This program and the accompanying materials are made
 // available under the terms of the Eclipse Public License 2.0
 // which is available at https://www.eclipse.org/legal/epl-2.0/
@@ -15,6 +15,9 @@ package devfileregistry
 import (
 	"fmt"
 
+	appsv1 "k8s.io/api/apps/v1"
+	corev1 "k8s.io/api/core/v1"
+
 	"github.com/eclipse-che/che-operator/pkg/common/chetypes"
 	"github.com/eclipse-che/che-operator/pkg/common/constants"
 	"github.com/eclipse-che/che-operator/pkg/deploy"
@@ -33,9 +36,18 @@ func NewDevfileRegistryReconciler() *DevfileRegistryReconciler {
 
 func (d *DevfileRegistryReconciler) Reconcile(ctx *chetypes.DeployContext) (reconcile.Result, bool, error) {
 	if ctx.CheCluster.Spec.Components.DevfileRegistry.DisableInternalRegistry {
-		ctx.CheCluster.Status.DevfileRegistryURL = ""
-		err := deploy.UpdateCheCRStatus(ctx, "DevfileRegistryURL", "")
-		return reconcile.Result{}, err == nil, err
+		_, _ = deploy.DeleteNamespacedObject(ctx, constants.DevfileRegistryName, &corev1.Service{})
+		_, _ = deploy.DeleteNamespacedObject(ctx, constants.DevfileRegistryName, &corev1.ConfigMap{})
+		_, _ = deploy.DeleteNamespacedObject(ctx, gateway.GatewayConfigMapNamePrefix+constants.DevfileRegistryName, &corev1.ConfigMap{})
+		_, _ = deploy.DeleteNamespacedObject(ctx, constants.DevfileRegistryName, &appsv1.Deployment{})
+
+		if ctx.CheCluster.Status.DevfileRegistryURL != "" {
+			ctx.CheCluster.Status.DevfileRegistryURL = ""
+			err := deploy.UpdateCheCRStatus(ctx, "DevfileRegistryURL", "")
+			return reconcile.Result{}, err == nil, err
+		}
+
+		return reconcile.Result{}, true, nil
 	}
 
 	done, err := d.syncService(ctx)
