@@ -16,6 +16,8 @@ import (
 	"fmt"
 	"strings"
 
+	"k8s.io/apimachinery/pkg/util/intstr"
+
 	identityprovider "github.com/eclipse-che/che-operator/pkg/deploy/identity-provider"
 	"github.com/sirupsen/logrus"
 	"k8s.io/apimachinery/pkg/api/resource"
@@ -97,6 +99,26 @@ skip_provider_button = false
 		skipAuthConfig(ctx.CheCluster))
 }
 
+func getSecretValue(ctx *chetypes.DeployContext) string {
+	secret := &corev1.Secret{}
+	exists, _ := deploy.GetNamespacedObject(ctx, ctx.CheCluster.Spec.Networking.Auth.OAuthSecret, secret)
+	if !exists {
+		// Kubernetes secret provided name not found. Assuming oAuthSecret provided is the actual secret.
+		return ctx.CheCluster.Spec.Networking.Auth.OAuthSecret
+	}
+
+	// Retrieve the value associated with the key "oAuthSecret"
+	value, found := secret.Data["oAuthSecret"]
+	if !found {
+		// Key 'oAuthSecret' not found. Assuming oAuthSecret provided is the actual secret.
+		return ctx.CheCluster.Spec.Networking.Auth.OAuthSecret
+	}
+
+	// Convert the byte slice to a string
+	secretValue := string(value)
+	return secretValue
+}
+
 func kubernetesOauthProxyConfig(ctx *chetypes.DeployContext, cookieSecret string) string {
 	return fmt.Sprintf(`
 proxy_prefix = "/oauth"
@@ -126,7 +148,7 @@ cookie_domains = "%s"
 		ctx.CheHost,
 		ctx.CheCluster.Spec.Networking.Auth.IdentityProviderURL,
 		ctx.CheCluster.Spec.Networking.Auth.OAuthClientName,
-		ctx.CheCluster.Spec.Networking.Auth.OAuthSecret,
+		getSecretValue(ctx),
 		cookieSecret,
 		cookieExpireAsString(ctx.CheCluster),
 		utils.Whitelist(ctx.CheHost),
@@ -185,9 +207,15 @@ func getOauthProxyContainerSpec(ctx *chetypes.DeployContext) corev1.Container {
 		Name:            "oauth-proxy",
 		Image:           defaults.GetGatewayAuthenticationSidecarImage(ctx.CheCluster),
 		ImagePullPolicy: corev1.PullIfNotPresent,
-		Args: []string{
-			"--config=/etc/oauth-proxy/oauth-proxy.cfg",
-		},
+		Args: map[bool][]string{
+			true: {
+				"--config=/etc/oauth-proxy/oauth-proxy.cfg",
+			},
+			false: {
+				"--config=/etc/oauth-proxy/oauth-proxy.cfg",
+				"--ping-path=/ping",
+				"--exclude-logging-path=/ping"},
+		}[infrastructure.IsOpenShift()],
 		VolumeMounts: []corev1.VolumeMount{
 			{
 				Name:      "oauth-proxy-config",
@@ -225,6 +253,40 @@ func getOauthProxyContainerSpec(ctx *chetypes.DeployContext) corev1.Container {
 				Value: configMapRevision,
 			},
 		},
+		ReadinessProbe: &corev1.Probe{
+			ProbeHandler: corev1.ProbeHandler{
+				HTTPGet: &corev1.HTTPGetAction{
+					Path: map[bool]string{true: "/oauth/healthz", false: "/ping"}[infrastructure.IsOpenShift()],
+					Port: intstr.IntOrString{
+						Type:   intstr.Int,
+						IntVal: int32(8080),
+					},
+					Scheme: corev1.URISchemeHTTP,
+				},
+			},
+			InitialDelaySeconds: 5,
+			TimeoutSeconds:      5,
+			PeriodSeconds:       5,
+			SuccessThreshold:    1,
+			FailureThreshold:    5,
+		},
+		LivenessProbe: &corev1.Probe{
+			ProbeHandler: corev1.ProbeHandler{
+				HTTPGet: &corev1.HTTPGetAction{
+					Path: map[bool]string{true: "/oauth/healthz", false: "/ping"}[infrastructure.IsOpenShift()],
+					Port: intstr.IntOrString{
+						Type:   intstr.Int,
+						IntVal: int32(8080),
+					},
+					Scheme: corev1.URISchemeHTTP,
+				},
+			},
+			InitialDelaySeconds: 15,
+			TimeoutSeconds:      5,
+			PeriodSeconds:       5,
+			SuccessThreshold:    1,
+			FailureThreshold:    5,
+		},
 	}
 }
 
